package de.mas.jnus.lib.implementations.wud.reader;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.util.Arrays;

import de.mas.jnus.lib.implementations.wud.WUDImage;

public class WUDDiscReaderSplitted extends WUDDiscReader{
    public WUDDiscReaderSplitted(WUDImage image) {
        super(image);
    }
    public static long WUD_SPLITTED_FILE_SIZE =  0x100000L * 0x800L;
    public static long NUMBER_OF_FILES =  12;
    public static String WUD_SPLITTED_DEFAULT_FILEPATTERN = "game_part%d.wud";
    
    @Override
    protected void readEncryptedToOutputStream(OutputStream outputStream, long offset, long size) throws IOException {
        RandomAccessFile input = getFileByOffset(offset);
       
        int bufferSize = 0x8000;
        byte[] buffer = new byte[bufferSize];
        long totalread = 0;
        long curOffset = offset;
        
        int part = getFilePartByOffset(offset);
        long offsetInFile = getOffsetInFilePart(part, curOffset);
                
        do{
            offsetInFile = getOffsetInFilePart(part, curOffset);
            int curReadSize = bufferSize;
            if((offsetInFile+bufferSize) >= WUD_SPLITTED_FILE_SIZE){ //Will we read above the part?
                long toRead =  WUD_SPLITTED_FILE_SIZE - offsetInFile; 
                if(toRead  == 0){ //just load the new file
                    input.close();
                    input = getFileByOffset(curOffset);
                    part++;
                    offsetInFile = getOffsetInFilePart(part, curOffset);
                }else{
                    curReadSize = (int) toRead; //And first only read until the part ends
                }
            }
            
            int read = input.read(buffer,0,curReadSize);
            if(read < 0) break;
            if(totalread + read > size){
                read = (int) (size - totalread);
            }
            try{
                outputStream.write(Arrays.copyOfRange(buffer, 0, read));
            }catch(IOException e){
                if(e.getMessage().equals("Pipe closed")){
                    break;
                }else{
                    input.close();
                    throw e;
                }
            }
            totalread += read;
            curOffset += read;
        }while(totalread < size);
        
        
        input.close();
        outputStream.close();
    }
    
    private int getFilePartByOffset(long offset){
        return (int) (offset/WUD_SPLITTED_FILE_SIZE)+1;
    }
    
    private long getOffsetInFilePart(int part,long offset){
        return  offset - ((long)(part-1) * WUD_SPLITTED_FILE_SIZE);
    }
    
    private RandomAccessFile getFileByOffset(long offset) throws IOException{
        File filehandlePart1 = getImage().getFileHandle();
        String pathToFiles = filehandlePart1.getParentFile().getAbsolutePath();
        
        int filePart = getFilePartByOffset(offset);
        
        String filePartPath = pathToFiles + File.separator + String.format(WUD_SPLITTED_DEFAULT_FILEPATTERN, filePart);
        
        File part = new File(filePartPath);
       
        if(!part.exists()){
            System.out.println("File does not exist");
            return null;
        }
        RandomAccessFile result =  new RandomAccessFile(part, "r");
        result.seek(getOffsetInFilePart(filePart, offset));
        return result;
    }
}
